'use strict';

const config = require('config');
const words = require('../../assets/mash-words.json');
let mash = null;

function start(length, context, attempt = 0) {
    const lengthWords = words[length];

    if (!lengthWords) {
	context.sendMessage(`No words with ${length} letters available`, context.room.id);

	return;
    }

    if (mash) {
	context.sendMessage(`The mash **${mash.anagram}** was not guessed, possible answers: ${mash.answers.map((answer) => `**${answer}**`).join(', ')}`, context.room.id);
	context.logger.info(`Mash '${mash.anagram}' discarded`);

	mash = null;
    }

    const wordEntries = Object.entries(lengthWords);
    const [key, answers] = wordEntries[Math.floor(Math.random() * wordEntries.length)];

    const anagram = key.split('').sort(() => Math.random() > .5 ? 1 : -1).join('');

    if (answers.includes(anagram)) {
	if (attempt >= 10) {
	    context.sendMessage(`Sorry, I did not find a mashable ${length}-letter word`);
	    return;
	}

	start(length, context, attempt + 1);

	return;
    }

    mash = { key, anagram, answers };

    context.sendMessage(`Stomp stomp, here's your mash: **${mash.anagram}**`, context.room.id);
    context.logger.info(`Mash started, '${anagram}' with answers ${answers.map((answer) => `'${answer}'`).join(', ')}`);
}

function play(word, context) {
    if (word.length !== mash.key.length) {
	context.sendMessage(`Your answer needs to be ${mash.key.length} letters, @${context.user.username}`, context.room.id);
	return;
    }

    const key = word.split('').sort().join('');

    if (key !== mash.key) {
	context.sendMessage(`You are not using the letters in **${mash.anagram}**, @${context.user.username}`, context.room.id);
	return;
    }

    if (word === mash.anagram) {
	context.sendMessage(`@${context.user.username}... :expressionless:`, context.room.id);
	return;
    }

    if (mash.answers.includes(word)) {
	context.sendMessage(mash.answers.length === 1
	    ? `**${word}** is the right answer, @${context.user.username} now has **${context.user.points + 1} ${context.user.points === 0 ? 'point' : 'points'}**! There were no other options for **${mash.anagram}**.`
	    : `**${word}** is the right answer, @${context.user.username} now has **${context.user.points + 1} ${context.user.points === 0 ? 'point' : 'points'}**! Other options for **${mash.anagram}**: ${mash.answers.filter((answer) => answer !== word).map((word) => `**${word}**`).join(', ')}`, context.room.id);

	context.logger.info(`Mash '${mash.anagram}' guessed by '${context.user.username}' with '${word}'`);
	context.setPoints(context.user, 1);

	mash = null;

	setTimeout(() => start(word.length, context), 2000);
    }
}

function onCommand(args, context) {
    const length = Number(args[0]);

    if (!Number.isNaN(length)) {
	start(length, context);
	return;
    }

    if (!args[0] && !mash) {
	context.sendMessage(`Start a mash with ${config.prefix}mash {length}`, context.room.id);
	return;
    }

    if (!args[0] && mash) {
	context.sendMessage(`The current mash is: **${mash.anagram}**`, context.room.id);
	return;
    }

    play(args[0], context);
}

module.exports = {
    name: 'Mash',
    onCommand,
};
